@@ -3,16 +3,42 @@ open! Import
 let invalid_argf = Printf.invalid_argf
 
 module T = struct
-  type t = bool [@@deriving_inline compare, enumerate, hash, sexp]
+  type t = bool [@@deriving_inline compare, enumerate, hash, sexp, sexp_grammar]
+
   let compare = (compare_bool : t -> t -> int)
-  let all = ([false; true] : t list)
-  let (hash_fold_t :
-         Ppx_hash_lib.Std.Hash.state -> t -> Ppx_hash_lib.Std.Hash.state) =
+  let all = ([ false; true ] : t list)
+
+  let (hash_fold_t : Ppx_hash_lib.Std.Hash.state -> t -> Ppx_hash_lib.Std.Hash.state) =
     hash_fold_bool
+
   and (hash : t -> Ppx_hash_lib.Std.Hash.hash_value) =
-    let func = hash_bool in fun x -> func x
+    let func = hash_bool in
+    fun x -> func x
+  ;;
+
   let t_of_sexp = (bool_of_sexp : Ppx_sexp_conv_lib.Sexp.t -> t)
   let sexp_of_t = (sexp_of_bool : t -> Ppx_sexp_conv_lib.Sexp.t)
+
+  let (t_sexp_grammar : Ppx_sexp_conv_lib.Sexp.Private.Raw_grammar.t) =
+    let (_the_generic_group : Ppx_sexp_conv_lib.Sexp.Private.Raw_grammar.generic_group) =
+      { implicit_vars = [ "bool" ]
+      ; ggid = "\146e\023\249\235eE\139c\132W\195\137\129\235\025"
+      ; types = [ "t", Implicit_var 0 ]
+      }
+    in
+    let (_the_group : Ppx_sexp_conv_lib.Sexp.Private.Raw_grammar.group) =
+      { gid = Ppx_sexp_conv_lib.Lazy_group_id.create ()
+      ; apply_implicit = [ bool_sexp_grammar ]
+      ; generic_group = _the_generic_group
+      ; origin = "bool.ml.T"
+      }
+    in
+    let (t_sexp_grammar : Ppx_sexp_conv_lib.Sexp.Private.Raw_grammar.t) =
+      Ref ("t", _the_group)
+    in
+    t_sexp_grammar
+  ;;
+
   [@@@end]
 
   let of_string = function
@@ -40,6 +66,7 @@ include Pretty_printer.Register (struct
    functions are available within this module. *)
 open! Bool_replace_polymorphic_compare
 
+let invariant (_ : t) = ()
 let between t ~low ~high = low <= t && t <= high
 let clamp_unchecked t ~min ~max = if t < min then min else if t <= max then t else max
 
